/*
    sudoku.cpp  -  Don Cross   http://cosinekitty.com

    Just what the world needs:  another program for solving Sudoku puzzles.
*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>


class SudokuBoard {
protected:
    struct square_info {
        char    row;
        char    col;
        char    numlegal;
        char    legaldigit [9];
    };

public:
    SudokuBoard()
    {
        reset();
    }

    void reset()
    {
        int row, col, digit;
        int rcb;    // (row=0,col=1,block=2) selector

        numEmptySquares = 81;
        numSolutionsFound = 0;

        for (row=0; row<9; ++row) {
            for (col=0; col<9; ++col) {
                square[row][col] = 0;   // designates empty square (1..9 when occupied)
            }
        }

        for (rcb=0; rcb<3; ++rcb) {
            for (col=0; col<9; ++col) {     // actually col is col/row/block, depending on value of rcb
                for (digit=0; digit<9; ++digit) {
                    available[rcb][col][digit] = true;      // quirk:  here digit is 0..8, for easy array indexing, but interpreted as 1..9.
                }
            }
        }
    }

    bool loadFromFile (const char *filename)
    {
        bool    success = false;
        int     row, col;
        char    line [16];

        reset();
        FILE *infile = fopen (filename, "rt");
        if (infile) {
            success = true;

            for (row=0; row<9; ++row) {
                if (!fgets(line,sizeof(line),infile)) {
                    fprintf (stderr, "Error:  cannot read line %d of file %s\n", 1+row, filename);
                    success = false;
                    break;
                }

                for (col=0; col<9; ++col) {
                    if (line[col]>='1' && line[col]<='9') {
                        drop (line[col] - '0', row, col);
                    } else if (line[col] != '.') {
                        fprintf (stderr, "Error:  line %d in file %s is not valid.\n", 1+row, filename);
                        success = false;
                        break;
                    }
                }
            }

            fclose (infile);
            infile = NULL;
        } else {
            fprintf (stderr, "Error:  Cannot open file '%s' for read.\n", filename);
        }
        return success;
    }

    void solve ()
    {
        int     row, col, digit, index;
        
        square_info     current, best;

        if (numEmptySquares == 0) {
            ++numSolutionsFound;
            printf("Solution #%d:\n", numSolutionsFound);
            dump();
        } else {
            best.numlegal = 10;     // impossible value; make sure first legal position replaces 'best'

            // first pass:  fill out si[]...
            for (row=0; row<9; ++row) {
                for (col=0; col<9; ++col) {
                    if (square[row][col] == 0) {
                        current.row = row;
                        current.col = col;
                        current.numlegal = 0;
                        for (digit=1; digit<=9; ++digit) {
                            if (islegal (digit, row, col)) {
                                current.legaldigit[current.numlegal++] = digit;
                            }
                        }
                        if (current.numlegal == 0) {
                            // we are stuck in a pickle... force immediate backtrack
                            return;
                        }
                        if (current.numlegal < best.numlegal) {
                            best = current;
                        }
                    }
                }
            }

            for (index=0; index < best.numlegal; ++index) {
                drop (best.legaldigit[index], best.row, best.col);
                solve();
                lift (best.row, best.col);
            }
        }
    }

    void dump()
    {
        for (int row=0; row<9; ++row) {
            for (int col=0; col<9; ++col) {
                if (square[row][col] == 0) {
                    printf(".");
                } else {
                    printf("%d", square[row][col]);
                }
            }
            printf("\n");
        }
        printf("\n");
    }

protected:
    void sanityCheck (int digit, int row, int col)
    {
        if (digit<1 || digit>9) {
            throw "SudokuBoard - digit is out of bounds";
        }

        if (row<0 || row>8) {
            throw "SudokuBoard - row is out of bounds";
        }

        if (col<0 || col>8) {
            throw "SudokuBoard - col is out of bounds";
        }
    }

    void drop (int digit, int row, int col)
    {
        sanityCheck (digit, row, col);
        int blk = calcblock (row, col);

        if (!available[0][row][digit-1] ||
            !available[1][col][digit-1] ||
            !available[2][blk][digit-1]) {
            fprintf (stderr, "!!! Illegal move attempted : digit=%d, row=%d, col=%d\n", digit, row, col);
            dump();
            throw "illegal move";
        }

        available[0][row][digit-1] = false;
        available[1][col][digit-1] = false;
        available[2][blk][digit-1] = false;

        square[row][col] = digit;

        --numEmptySquares;
    }

    void lift (int row, int col)
    {
        int digit = square[row][col];
        sanityCheck (digit, row, col);
        int blk = calcblock (row, col);

        available[0][row][digit-1] = true;
        available[1][col][digit-1] = true;
        available[2][blk][digit-1] = true;

        square[row][col] = 0;

        ++numEmptySquares;
    }

    bool islegal (int digit, int row, int col)
    {
        sanityCheck (digit, row, col);
        int blk = calcblock (row, col);
        return 
            available[0][row][digit-1] &&
            available[1][col][digit-1] &&
            available[2][blk][digit-1]
        ;
    }

    static int calcblock (int row, int col)
    {
        return (col/3) + 3*(row/3);
    }

private:
    bool    available [3] [9] [9];      // [0=row,1=col,2=block] [position] [digit]
    int     square [9] [9];             // [row] [col]
    int     numEmptySquares;
    int     numSolutionsFound;
};



int main (int argc, const char *argv[])
{
    int rc = 1;

    printf ("Sudoku puzzle solver - by Don Cross  http://cosinekitty.com\n\n");
    if (argc == 2) {
        const char *filename = argv[1];
        try {
            SudokuBoard     board;
            if (board.loadFromFile (filename)) {
                printf("The following puzzle was loaded:\n");
                board.dump();
                board.solve();
            }
        } catch (const char *error) {
            fprintf (stderr, "!!! EXCEPTION:  %s\n", error);
            rc = 2;
        }
    } else {
        const char *help = 
            "Use:  sudoku puzzlefile\n"
            "\n"
            "where puzzlefile is a text file containing 9 lines of text,\n"
            "each line having 9 characters which are digits 1-9, or '.'\n"
            "for representing empty squares.\n"
            "\n"
            "Here is an example of valid puzzlefile contents:\n"
            "....9...6\n"
            "..3.2.758\n"
            "...5....4\n"
            "2..8..9..\n"
            ".1.....7.\n"
            "..9..4..2\n"
            "6....9...\n"
            "842.3.5..\n"
            "1...4....\n"
            "\n";

        printf ("%s", help);
    }

    return rc;
}


/*
    $Log: sudoku.cpp,v $
    Revision 1.4  2006/11/07 22:53:41  dcross
    1. Removed node counter for search (don't see the point any more).
    2. Added brag text.
    3. Expanded help text, describing input file format.

    Revision 1.3  2006/11/07 22:44:02  dcross
    Yay!  I got the solver working so well that it finishes in a fraction of a second!
    My mistake was doing both iteration through all possible legal moves at each level
    of recursion, instead of just picking the square with the fewest legal moves, making
    that single move, then recursing.  The old way was re-examining the same legal moves
    trillions of times, whereas the new approach homes in on the solution almost immediately.

    Revision 1.2  2006/11/06 18:26:33  dcross
    Still have not seen this program find a solution yet, but I'm hoping this modification is a step
    in the right direction:  now traverse in ascending order of how many legal moves there are.
    Still, it would seem that the performance problem has been made worse because we are
    generating billions of redundant positions.  Need to figure out a way to try a given position only once.

    Revision 1.1  2006/11/06 01:16:51  dcross
    Not sure this program really works yet, because it hasn't yet solved the puzzle Bryan gave me.
    I'm checking it in just because it seems to be the correct algorithm, yet it sure could use some optimization.
    I'm going to let it run all night and check in the morning to see if it printed any solutions.

*/